import { ApiAuthenticationConfig, ApiConfig, Query, RequestOptions, RequestType, ResponseType, ResultItemTupleTypes } from './types';
export declare const DEFAULT_BASE_URL = "https://rest.spinque.com/";
/**
 * Send queries to the Spinque Query API using fetch.
 */
export declare class Api {
    /**
     * URL to the Spinque Query API deployment.
     *
     * @default https://rest.spinque.com/
     */
    baseUrl: string;
    /**
     * Version of the Spinque Query API deployment.
     *
     * @default 4
     */
    version?: string | undefined;
    /**
     * Name of the Spinque workspace that should be addressed.
     * The Spinque Desk administrator working on your project knowns this value.
     */
    workspace?: string;
    /**
     * Name of the API that is provided by the workspace.
     * The Spinque Desk administrator working on your project knowns this value.
     */
    api?: string;
    /**
     * Name of the configuration of the Spinque workspace that should be used.
     * Usually, this is something like 'production', 'development' or 'default'.
     * The Spinque Desk administrator working on your project knowns this value.
     *
     * @default default
     */
    config?: string | undefined;
    private _authentication?;
    private _authenticator?;
    private _isInitialized;
    constructor(apiConfig?: ApiConfig);
    get accessToken(): string | undefined;
    /**
     * Getter for authentication configuration
     */
    get authentication(): ApiAuthenticationConfig | undefined;
    /**
     * Getter for authentication configuration
     */
    get apiConfig(): ApiConfig;
    /**
     * Fetch a Query (or array of Queries). Takes optional RequestOptions and RequestType into account.
     * Optionally the `fetch` RequestInit can be passed.
     */
    fetch<T extends ResultItemTupleTypes[] = ResultItemTupleTypes[], R extends RequestType = 'results'>(queries: Query | Query[], options?: RequestOptions, requestType?: R, requestInit?: RequestInit): Promise<ResponseType<R, T>>;
    /**
     * Handle the response of a fetch to Spinque Query API.
     */
    private handleResponse;
}
