"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Api = exports.DEFAULT_BASE_URL = void 0;
var authentication_1 = require("./authentication");
var types_1 = require("./types");
var utils_1 = require("./utils");
// This is the default base URL to the Spinque Query API.
exports.DEFAULT_BASE_URL = 'https://rest.spinque.com/';
/**
 * Send queries to the Spinque Query API using fetch.
 */
var Api = /** @class */ (function () {
    function Api(apiConfig) {
        var _this = this;
        /**
         * URL to the Spinque Query API deployment.
         *
         * @default https://rest.spinque.com/
         */
        this.baseUrl = exports.DEFAULT_BASE_URL;
        /**
         * Version of the Spinque Query API deployment.
         *
         * @default 4
         */
        this.version = '4';
        /**
         * Name of the configuration of the Spinque workspace that should be used.
         * Usually, this is something like 'production', 'development' or 'default'.
         * The Spinque Desk administrator working on your project knowns this value.
         *
         * @default default
         */
        this.config = 'default';
        if (apiConfig && apiConfig.baseUrl) {
            this.baseUrl = apiConfig.baseUrl;
        }
        if (apiConfig && apiConfig.version) {
            this.version = apiConfig.version;
        }
        if (apiConfig && apiConfig.workspace) {
            this.workspace = apiConfig.workspace;
        }
        if (apiConfig && apiConfig.api) {
            this.api = apiConfig.api;
        }
        if (apiConfig && apiConfig.config) {
            this.config = apiConfig.config;
        }
        if (apiConfig && apiConfig.authentication) {
            if (apiConfig.authentication.type === 'client-credentials') {
                this._authentication = apiConfig.authentication;
                this._authenticator = new authentication_1.ClientCredentials(apiConfig.authentication.clientId, apiConfig.authentication.clientSecret, apiConfig.authentication.tokenCache, apiConfig.authentication.authServer, apiConfig.baseUrl);
            }
            if (apiConfig.authentication.type === 'pkce') {
                this._authentication = apiConfig.authentication;
                this._authenticator = new authentication_1.PKCE(apiConfig.authentication.clientId, apiConfig.authentication.callback, apiConfig.authentication.authServer, apiConfig.authentication.tokenCache, apiConfig.baseUrl);
            }
        }
        // If auth is configured but there is not token yet..
        if ((apiConfig === null || apiConfig === void 0 ? void 0 : apiConfig.authentication) &&
            (!apiConfig.authentication.tokenCache || !apiConfig.authentication.tokenCache.get())) {
            // Request the API information to see if auth is actually needed
            var url = (0, utils_1.apiUrl)(this.apiConfig);
            // Save the status in _isInitialized to delay incoming fetch requests until we know if auth is needed
            this._isInitialized = fetch(url).then(function (res) {
                var _a;
                if (res.status === 200 || !_this._authenticator) {
                    // If this is allowed without authentication, we can forget about the auth confug
                    _this._authentication = undefined;
                    return new Promise(function (resolve) { return resolve(true); });
                }
                else {
                    // If this is not allowed, request an access token
                    return (_a = _this._authenticator) === null || _a === void 0 ? void 0 : _a.accessToken.then(function () { return true; });
                }
            });
        }
        else {
            this._isInitialized = new Promise(function (resolve) { return resolve(true); });
        }
    }
    Object.defineProperty(Api.prototype, "accessToken", {
        get: function () {
            var _a;
            return (_a = this._authenticator) === null || _a === void 0 ? void 0 : _a._accessToken;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Api.prototype, "authentication", {
        /**
         * Getter for authentication configuration
         */
        get: function () {
            return this._authentication;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Api.prototype, "apiConfig", {
        /**
         * Getter for authentication configuration
         */
        get: function () {
            return {
                workspace: this.workspace,
                version: this.version,
                baseUrl: this.baseUrl,
                config: this.config,
                api: this.api,
                authentication: this.authentication,
            };
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Fetch a Query (or array of Queries). Takes optional RequestOptions and RequestType into account.
     * Optionally the `fetch` RequestInit can be passed.
     */
    Api.prototype.fetch = function (queries, options, requestType, requestInit) {
        if (requestInit === void 0) { requestInit = {}; }
        return __awaiter(this, void 0, void 0, function () {
            var url, token;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._isInitialized];
                    case 1:
                        _a.sent();
                        // Convert single query to array of queries
                        if (!(queries instanceof Array)) {
                            queries = [queries];
                        }
                        if (queries.length === 0) {
                            throw new Error('Queries array is empty');
                        }
                        url = (0, utils_1.urlFromQueries)(this.apiConfig, queries, options, requestType);
                        if (!(this.authentication && this._authenticator)) return [3 /*break*/, 3];
                        return [4 /*yield*/, this._authenticator.accessToken];
                    case 2:
                        token = _a.sent();
                        requestInit = { headers: new Headers(__assign(__assign({}, requestInit.headers), { Authorization: "Bearer ".concat(token) })) };
                        _a.label = 3;
                    case 3: 
                    // Make the request
                    return [2 /*return*/, fetch(url, requestInit).then(function (res) { return _this.handleResponse(res); })];
                }
            });
        });
    };
    /**
     * Handle the response of a fetch to Spinque Query API.
     */
    Api.prototype.handleResponse = function (response) {
        return __awaiter(this, void 0, void 0, function () {
            var json;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, response.json()];
                    case 1:
                        json = _a.sent();
                        if (response.status === 200) {
                            return [2 /*return*/, json];
                        }
                        if (response.status === 401) {
                            throw new types_1.UnauthorizedError(json.message, 401);
                        }
                        if (response.status === 400 && json.message.includes('no endpoint')) {
                            throw new types_1.EndpointNotFoundError(json.message, 400);
                        }
                        if (response.status === 500) {
                            throw new types_1.ServerError(json.message, 401);
                        }
                        if (response.status === 404 && json.message.includes('No such api')) {
                            throw new types_1.ApiNotFoundError(json.message, 404);
                        }
                        if (response.status === 404 && json.message.includes('No such workspace configuration')) {
                            throw new types_1.WorkspaceConfigNotFoundError(json.message, 404);
                        }
                        throw new types_1.ErrorResponse('Unknown error: ' + (json.message || ''), response.status);
                }
            });
        });
    };
    return Api;
}());
exports.Api = Api;
//# sourceMappingURL=Api.js.map