import { TokenCache } from './TokenCache';
export declare const DEFAULT_AUTH_SERVER = "https://login.spinque.com/";
export declare const DEFAULT_AUDIENCE = "https://rest.spinque.com/";
/**
 * Abstract class with utitily functions for working with access tokens such as storage
 */
export declare abstract class Authenticator {
    private _tokenCache;
    _authInProgress: boolean;
    _accessToken?: string;
    _expires?: number;
    constructor(_tokenCache: TokenCache);
    /**
     * A Promise that delays any operation until an access token is set (with intervals of 50ms)
     * This is used to delay incoming request from our app when an access token is already
     * being requested but has not yet been received.
     */
    private _waitForAccessToken;
    /**
     * Promise that will resolve with an access token if available or undefined otherwise.
     * This will try to fetch a new access token if:
     *  - the class implements the fetchAccessToken method
     *  - no unexpired access token is stored in this class
     */
    get accessToken(): Promise<string | undefined>;
    /**
     * Puts an access token and the expiration time in storage for usage when calling Spinque Query API
     */
    setAccessToken(accessToken: string, expiresIn: number): void;
    /**
     * Abstract method that must be implemented by extending classes for specific OAuth 2.0 grants/flows.
     */
    abstract fetchAccessToken(): Promise<{
        accessToken: string;
        expiresIn: number;
    } | undefined>;
}
