"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.localStorageTokenCache = void 0;
/**
 * Implementation of TokenCache that uses localStorage to cache tokens.
 */
exports.localStorageTokenCache = {
    get: function () {
        if (typeof window === 'undefined') {
            return null;
        }
        try {
            var accessToken = localStorage.getItem('@spinque/query-api/access-token');
            var expires = parseInt(localStorage.getItem('@spinque/query-api/expires') || '', 10);
            if (accessToken && expires && expires > Date.now() + 1000) {
                return { accessToken: accessToken, expires: expires };
            }
            else {
                localStorage.removeItem('@spinque/query-api/access-token');
                localStorage.removeItem('@spinque/query-api/expires');
                return null;
            }
        }
        catch (error) {
            return null;
        }
    },
    set: function (token, expires) {
        if (typeof window === 'undefined') {
            return;
        }
        localStorage.setItem("@spinque/query-api/access-token", token);
        localStorage.setItem("@spinque/query-api/expires", "".concat(expires));
    },
};
//# sourceMappingURL=TokenCache.js.map