"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isCluster = exports.getClusters = void 0;
var utils_1 = require("./utils");
var DEFAULT_CLUSTER_ENDPOINT = 'type:FILTER';
var DEFAULT_CLUSTER_PARAMETER_NAME = 'value';
var DEFAULT_CLUSTER_PARAMETER_TYPE = 'TUPLE_LIST';
var RDFS_CLASS = 'http://www.w3.org/2000/01/rdf-schema#Class';
/**
 * Taskes a ResultsResponse, finds clusters in the results and returns a list of Cluster objects.
 * These Cluster objects contain a Query that can be used to fetch the contents of the clusters
 * by stacking the Query on top of the results Query[].
 */
var getClusters = function (results, options) {
    var e_1, _a, _b;
    var _c, _d, _e;
    if (options === void 0) { options = {}; }
    var endpoint = (_c = options.clusterEndpoint) !== null && _c !== void 0 ? _c : DEFAULT_CLUSTER_ENDPOINT;
    var parameterName = (_d = options.clusterParameterName) !== null && _d !== void 0 ? _d : DEFAULT_CLUSTER_PARAMETER_NAME;
    var parameterType = (_e = options.clusterParameterType) !== null && _e !== void 0 ? _e : DEFAULT_CLUSTER_PARAMETER_TYPE;
    var clusters = [];
    try {
        // Loop through result items and when encountering a cluster, add it to the list
        for (var _f = __values(results.items), _g = _f.next(); !_g.done; _g = _f.next()) {
            var item = _g.value;
            if (!(0, exports.isCluster)(item)) {
                continue;
            }
            var cluster = {
                probability: item.probability,
                rank: item.rank,
                class: item.tuple[0].class,
                query: {
                    endpoint: endpoint,
                    parameters: (_b = {},
                        _b[parameterName] = parameterType === 'TUPLE_LIST' ? (0, utils_1.tupleListToString)([item.tuple[0].id]) : item.tuple[0].id,
                        _b),
                },
            };
            clusters.push(cluster);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_g && !_g.done && (_a = _f.return)) _a.call(_f);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return clusters;
};
exports.getClusters = getClusters;
/**
 * Takes a result item and checks if it is a cluster.
 * A result is considered a cluster when it's of type rdfs:Class.
 */
var isCluster = function (item) {
    if (!item.tuple ||
        item.tuple.length !== 1 ||
        typeof item.tuple[0] === 'string' ||
        typeof item.tuple[0] === 'number') {
        return false;
    }
    return item.tuple[0].class.includes(RDFS_CLASS);
};
exports.isCluster = isCluster;
//# sourceMappingURL=clusters.js.map