export declare type Language = 'english' | 'dutch';
export interface SnippetOptions {
    /**
     * Maximum number of characters the snippet may be.
     */
    size?: number;
    /**
     * Whether to highlight terms using <mark> elements. Defaults to false.
     */
    highlight?: boolean;
    /**
     * Lists of stopwords to ignore while finding the best snippet.
     */
    stopwords?: Language;
    /**
     * Stemmer to use when comparing query terms with text terms.
     */
    stemmer?: Language;
}
/**
 * Takes a piece of text and a text query and returns a snippet of the text that best matches the query.
 */
export declare const getSnippet: (text: string, query: string, options?: SnippetOptions) => string;
/**
 * Truncates a piece of text to the desired maximum length and appends '...' to the end.
 */
export declare const truncate: (text: string, size?: number) => string;
/**
 * Split text into sentence using a Regular Expression.
 */
export declare const splitIntoSentences: (text: string) => string[];
