"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var FacetedSearch_1 = require("../FacetedSearch");
describe('FacetedSearch', function () {
    it('should be constructable with only a searchQuery', function () {
        var sq = { endpoint: 'my-endpoint', parameters: { q: '' } };
        var fs = new FacetedSearch_1.FacetedSearch(sq);
        expect(fs).toBeDefined();
    });
    it('should be able to set facets', function () {
        var sq = { endpoint: 'my-endpoint', parameters: { q: '' } };
        var fs = new FacetedSearch_1.FacetedSearch(sq);
        fs.addFacet('genre');
        expect(fs.facets).toBeDefined();
        expect(fs.facets.length).toEqual(1);
    });
    it('should create single-select facets by default', function () {
        var sq = { endpoint: 'my-endpoint', parameters: { q: '' } };
        var fs = new FacetedSearch_1.FacetedSearch(sq);
        fs.addFacet('genre');
        expect(fs.facets[0].type).toEqual('single');
    });
    it('should set the facet value using setFacetSelection for single-select', function () {
        var sq = { endpoint: 'my-endpoint', parameters: { q: '' } };
        var fs = new FacetedSearch_1.FacetedSearch(sq);
        fs.addFacet('genre');
        fs.setFacetSelection('genre', 'a');
        expect(fs.facets[0].filterParameterValue).toEqual('a');
    });
    it('should set the facet value using setFacetSelection for multiple-select', function () {
        var sq = { endpoint: 'my-endpoint', parameters: { q: '' } };
        var fs = new FacetedSearch_1.FacetedSearch(sq);
        fs.addFacet('genre', FacetedSearch_1.FacetType.multiple);
        fs.setFacetSelection('genre', ['a', 'b']);
        expect(fs.facets[0].filterParameterValue).toEqual('1(a)|1(b)');
    });
    it('should not allow multiple values to be selected for single-select facets', function () {
        var sq = { endpoint: 'my-endpoint', parameters: { q: '' } };
        var fs = new FacetedSearch_1.FacetedSearch(sq);
        fs.addFacet('genre');
        expect(function () { return fs.setFacetSelection('genre', ['a', 'b']); }).toThrow();
    });
    it('should create mulitple-select facets on request', function () {
        var sq = { endpoint: 'my-endpoint', parameters: { q: '' } };
        var fs = new FacetedSearch_1.FacetedSearch(sq);
        fs.addFacet('genre', FacetedSearch_1.FacetType.multiple);
        expect(fs.facets[0].type).toEqual(FacetedSearch_1.FacetType.multiple);
    });
});
//# sourceMappingURL=FacetedSearch.spec.js.map