"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServerError = exports.UnauthorizedError = exports.EndpointNotFoundError = exports.ApiNotFoundError = exports.WorkspaceConfigNotFoundError = exports.ErrorResponse = void 0;
/**
 * Generic error response class. Is implemented by more specific error type classes.
 */
var ErrorResponse = /** @class */ (function () {
    function ErrorResponse(message, status) {
        this.message = message;
        this.status = status;
    }
    return ErrorResponse;
}());
exports.ErrorResponse = ErrorResponse;
/**
 * Error class used when Spinque cannot find the workspace configuration you requested.
 * The workspace or configuration might be misspelled or removed.
 */
var WorkspaceConfigNotFoundError = /** @class */ (function () {
    function WorkspaceConfigNotFoundError(message, status) {
        this.message = message;
        this.status = status;
    }
    return WorkspaceConfigNotFoundError;
}());
exports.WorkspaceConfigNotFoundError = WorkspaceConfigNotFoundError;
/**
 * Error class used when Spinque cannot find the API you requested.
 * The API might be misspelled or removed.
 */
var ApiNotFoundError = /** @class */ (function () {
    function ApiNotFoundError(message, status) {
        this.message = message;
        this.status = status;
    }
    return ApiNotFoundError;
}());
exports.ApiNotFoundError = ApiNotFoundError;
/**
 * Error class used when Spinque cannot find the endpoint you requested.
 * The endpoint might be misspelled or removed.
 */
var EndpointNotFoundError = /** @class */ (function () {
    function EndpointNotFoundError(message, status) {
        this.message = message;
        this.status = status;
    }
    return EndpointNotFoundError;
}());
exports.EndpointNotFoundError = EndpointNotFoundError;
/**
 * Error class used when you are not authorized to request results for
 * this workspace, API or endpoint.
 */
var UnauthorizedError = /** @class */ (function () {
    function UnauthorizedError(message, status) {
        this.message = message;
        this.status = status;
    }
    return UnauthorizedError;
}());
exports.UnauthorizedError = UnauthorizedError;
/**
 * Error class when something fails on the side of Spinque. Please contact
 * your system administrator when this happens.
 */
var ServerError = /** @class */ (function () {
    function ServerError(message, status) {
        this.message = message;
        this.status = status;
    }
    return ServerError;
}());
exports.ServerError = ServerError;
//# sourceMappingURL=types.js.map