"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isBrowser = exports.stringifyQueries = exports.parseQueries = exports.join = exports.tupleListToString = exports.stringToTupleList = exports.urlFromQueries = exports.apiStatusUrl = exports.apiUrl = exports.pathFromQuery = exports.pathFromQueries = void 0;
/**
 * Takes an array of Query objects and returns the path they would represent in a Query API request URL.
 */
var pathFromQueries = function (queries) {
    return exports.join.apply(void 0, __spreadArray([], __read(queries.map(exports.pathFromQuery)), false));
};
exports.pathFromQueries = pathFromQueries;
/**
 * Takes a Query and returns the path it would represent in a Query API request URL.
 */
var pathFromQuery = function (query) {
    var parts = ['e', encodeURIComponent(query.endpoint)];
    if (query.parameters) {
        Object.entries(query.parameters).forEach(function (_a) {
            var _b = __read(_a, 2), name = _b[0], value = _b[1];
            parts.push('p', encodeURIComponent(name), encodeURIComponent(value));
        });
    }
    return exports.join.apply(void 0, __spreadArray([], __read(parts), false));
};
exports.pathFromQuery = pathFromQuery;
/**
 * Takes an ApiConfig object and returns the URL to fetch API details
 */
var apiUrl = function (config) {
    if (!config.baseUrl) {
        throw new Error('Base URL missing');
    }
    if (!config.version) {
        throw new Error('Version missing');
    }
    if (!config.workspace) {
        throw new Error('Workspace missing');
    }
    if (!config.api) {
        throw new Error('API name missing');
    }
    var url = config.baseUrl;
    if (!url.endsWith('/')) {
        url += '/';
    }
    // Construct base URL containing Spinque version and workspace
    url += (0, exports.join)(config.version, config.workspace, 'api', config.api);
    // For loadbalancer reasons, the API URL should end with a slash
    if (!url.endsWith('/')) {
        url += '/';
    }
    // Add config if provided
    if (config.config) {
        url += "?config=".concat(config.config);
    }
    return url;
};
exports.apiUrl = apiUrl;
/**
 * Takes an ApiConfig object and returns the URL to fetch API status
 */
var apiStatusUrl = function (config) {
    if (!config.baseUrl) {
        throw new Error('Base URL missing');
    }
    if (!config.version) {
        throw new Error('Version missing');
    }
    if (!config.workspace) {
        throw new Error('Workspace missing');
    }
    if (!config.api) {
        throw new Error('API name missing');
    }
    var url = config.baseUrl;
    if (!url.endsWith('/')) {
        url += '/';
    }
    // Construct base URL containing Spinque version and workspace
    url += (0, exports.join)(config.version, config.workspace, 'api', config.api, 'status');
    // Add config if provided
    if (config.config) {
        url += "?config=".concat(config.config);
    }
    return url;
};
exports.apiStatusUrl = apiStatusUrl;
/**
 * Takes an ApiConfig object and array of Query objects and returns a Query API request URL.
 */
var urlFromQueries = function (config, queries, options, requestType) {
    if (requestType === void 0) { requestType = 'results'; }
    if (!(queries instanceof Array)) {
        queries = [queries];
    }
    if (!config.baseUrl) {
        throw new Error('Base URL missing');
    }
    if (!config.version) {
        throw new Error('Version missing');
    }
    if (!config.workspace) {
        throw new Error('Workspace missing');
    }
    if (!config.api) {
        throw new Error('API name missing');
    }
    var url = config.baseUrl;
    if (!url.endsWith('/')) {
        url += '/';
    }
    // Construct base URL containing Spinque version and workspace
    url += (0, exports.join)(config.version, config.workspace, 'api', config.api);
    // Add the path represented by the Query objects and request type
    url += '/' + (0, exports.join)((0, exports.pathFromQueries)(queries), requestType);
    // Add config if provided
    if (config.config) {
        url += "?config=".concat(config.config);
    }
    if (options && Object.keys(options).length > 0) {
        Object.entries(options).forEach(function (_a, index) {
            var _b = __read(_a, 2), option = _b[0], value = _b[1];
            if (index === 0 && !config.config) {
                url += '?';
            }
            else {
                url += '&';
            }
            url += "".concat(option, "=").concat(value);
        });
    }
    return url;
};
exports.urlFromQueries = urlFromQueries;
/**
 * Given a tuple list (and optionally scores), return a string representation.
 */
var stringToTupleList = function (value) {
    try {
        return value.split('|').reduce(function (acc, cur) {
            var score = parseFloat(cur.split('(')[0]);
            var tuples = cur.split('(')[1].split(')')[0].split(',');
            acc.scores.push(score);
            acc.tuples.push(tuples);
            return acc;
        }, { scores: [], tuples: [] });
    }
    catch (error) {
        return null;
    }
};
exports.stringToTupleList = stringToTupleList;
/**
 * Given a string, try to parse as tuple list.
 */
var tupleListToString = function (
// tuples can be either a string, a number, an array of strings or numbers,
// or an array of arrays of strings or numbers
tuples, scores) {
    var _tuples = ensureTupleList(tuples);
    if (scores && scores.length !== _tuples.length) {
        throw new Error('Scores does not contain as many items as tuples');
    }
    var _scores = scores || Array.from(Array(_tuples.length)).map(function () { return 1; });
    return _tuples
        .map(function (tuple, index) {
        var s = _scores[index];
        var values = tuple.join(',');
        return "".concat(s, "(").concat(values, ")");
    })
        .join('|');
};
exports.tupleListToString = tupleListToString;
/**
 * Takes a value that should be a tuple list and ensures it has a normalized form.
 */
var ensureTupleList = function (value) {
    var e_1, _a;
    // Convert string or number to nested array
    if (typeof value === 'string' || typeof value === 'number') {
        return [[value]];
    }
    if (!(value instanceof Array)) {
        throw new Error('Tuple list should be of type: (string|number)[][] | (string|number)[] | (string|number)');
    }
    if (value.length === 0) {
        return [[]];
    }
    var someAreArrays = false;
    var allAreArrays = true;
    try {
        for (var value_1 = __values(value), value_1_1 = value_1.next(); !value_1_1.done; value_1_1 = value_1.next()) {
            var t = value_1_1.value;
            if (t instanceof Array) {
                someAreArrays = true;
                if (value[0] instanceof Array && t.length !== value[0].length) {
                    throw new Error('Tuple list has unequally sized rows (some have more columns)');
                }
            }
            else {
                allAreArrays = false;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (value_1_1 && !value_1_1.done && (_a = value_1.return)) _a.call(value_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    if (someAreArrays && !allAreArrays) {
        throw new Error('Tuple list has unequally sized rows (some are a single value, some arrays)');
    }
    if (!someAreArrays) {
        return value.map(function (v) { return [v]; });
    }
    return value;
};
/**
 * Joins together URL parts into an URL
 */
var join = function () {
    var e_2, _a;
    var segments = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        segments[_i] = arguments[_i];
    }
    var parts = segments.reduce(function (_parts, segment) {
        // Remove leading slashes from non-first part.
        if (_parts.length > 0) {
            segment = segment.replace(/^\//, '');
        }
        // Remove trailing slashes.
        segment = segment.replace(/\/$/, '');
        return _parts.concat(segment.split('/'));
    }, []);
    var resultParts = [];
    try {
        for (var parts_1 = __values(parts), parts_1_1 = parts_1.next(); !parts_1_1.done; parts_1_1 = parts_1.next()) {
            var part = parts_1_1.value;
            if (part === '.') {
                continue;
            }
            if (part === '..') {
                resultParts.pop();
                continue;
            }
            resultParts.push(part);
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (parts_1_1 && !parts_1_1.done && (_a = parts_1.return)) _a.call(parts_1);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return resultParts.join('/');
};
exports.join = join;
/**
 * Expects a string generated by stringifyQueries and returns an array of Query's
 */
var parseQueries = function (stringified) {
    if (!stringified) {
        return [];
    }
    try {
        var endpoints = JSON.parse(stringified);
        return endpoints.map(function (e) {
            if (typeof e === 'string') {
                return {
                    endpoint: e,
                    parameters: undefined,
                };
            }
            else {
                return {
                    endpoint: e[0],
                    parameters: e[1],
                };
            }
        });
    }
    catch (error) {
        return [];
    }
};
exports.parseQueries = parseQueries;
/**
 * Expects an array of Query's and turns them into a string
 */
var stringifyQueries = function (queries) {
    var endpointString = queries.map(function (q) { return (q.parameters ? [q.endpoint, q.parameters] : q.endpoint); });
    return JSON.stringify(endpointString);
};
exports.stringifyQueries = stringifyQueries;
exports.isBrowser = typeof window !== 'undefined' && typeof window.document !== 'undefined';
//# sourceMappingURL=utils.js.map